// Include files
#include "CaloChi22ID.h"

// =============================================================================
/** @file
 *  Implementation file for class CaloChi22ID
 *  @date 2006-06-18
 *  @author Vanya BELYAEV ibelyaev@physics.syr.edu
 */
// =============================================================================

DECLARE_COMPONENT( CaloChi22ID )

// =============================================================================
/// Standard protected constructor
// =============================================================================

CaloChi22ID::CaloChi22ID(const std::string& name, ISvcLocator* pSvcLocator)
    : CaloTrackAlg(name, pSvcLocator) {
  // context-dependent default track container
  m_tracks.clear();
  m_tracks = LHCb::CaloAlgUtils::TrackLocations(context());
}

// =============================================================================
/// algorithm execution
// =============================================================================

StatusCode CaloChi22ID::execute() {
  typedef LHCb::Calo2Track::ITrHypoTable TABLEI;
  typedef LHCb::Relation1D<LHCb::Track, float> Table;
  // get the input table
  // get the input data
  if (!exist<TABLEI>(m_input))
    return Warning(" Input missing '" + m_input + "'", StatusCode::SUCCESS);
  const TABLEI* input = get<TABLEI>(m_input);
  // create and register the output table
  Table* output = new Table(input->relations().size() + 10);
  put(output, m_output);
  // perform the actual jobs
  StatusCode sc = doTheJob(input, output);
  if (counterStat->isQuiet()) counter(Gaudi::Utils::toString(m_tracks.value()) + "->"
            + m_input + "=>" + m_output) += output->i_relations().size();
  return sc;
}

// =============================================================================
