// Include files
#include "CaloID2DLL.h"

// ============================================================================
/** @class EcalPIDmuAlg  EcalPIDmuAlg.cpp
 *  The preconfigured instance of class CaloID2DLL
 *  @author Vanya BELYAEV ibelyaev@physics.syr.edu
 *  @date 2006-06-18
 */
// ============================================================================

class EcalPIDmuAlg final : public CaloID2DLL {
 public:
  EcalPIDmuAlg(const std::string& name, ISvcLocator* pSvc)
      : CaloID2DLL(name, pSvc) {
    using LHCb::CaloAlgUtils::CaloIdLocation;

    Gaudi::Functional::updateHandleLocation(
        *this, "Input", CaloIdLocation("EcalE", context()));
    Gaudi::Functional::updateHandleLocation(
        *this, "Output", CaloIdLocation("EcalPIDmu", context()));

    _setProperty("nVlong"  , Gaudi::Utils::toString(5 * Gaudi::Units::GeV));
    _setProperty("nVdown"  , Gaudi::Utils::toString(5 * Gaudi::Units::GeV));
    _setProperty("nVTtrack", Gaudi::Utils::toString(5 * Gaudi::Units::GeV));
    _setProperty("nMlong"  , Gaudi::Utils::toString(25 * Gaudi::Units::GeV));
    _setProperty("nMdown"  , Gaudi::Utils::toString(25 * Gaudi::Units::GeV));
    _setProperty("nMTtrack", Gaudi::Utils::toString(25 * Gaudi::Units::GeV));

    _setProperty("HistogramL"   , "DLL_Long");
    _setProperty("HistogramD"   , "DLL_Downstream");
    _setProperty("HistogramT"   , "DLL_Ttrack");
    _setProperty("ConditionName", "Conditions/ParticleID/Calo/EcalPIDmu");

    _setProperty("HistogramL_THS", "CaloPIDs/CALO/ECALPIDM/h3");
    _setProperty("HistogramD_THS", "CaloPIDs/CALO/ECALPIDM/h5");
    _setProperty("HistogramT_THS", "CaloPIDs/CALO/ECALPIDM/h6");

    _setProperty("AcceptedType", Gaudi::Utils::toString<int>(
                                     LHCb::Track::Types::Long, LHCb::Track::Types::Ttrack,
                                     LHCb::Track::Types::Downstream));
  };
};

// ============================================================================

DECLARE_COMPONENT( EcalPIDmuAlg )

// ============================================================================
