// ============================================================================
// Include files 
// ============================================================================
// local
// ============================================================================
#include "CaloSelectorOR.h"
// ============================================================================
/** @file 
 * 
 *  Implementation file for class : CaloSelectorOR
 * 
 *  @author Vanya Belyaev Ivan.Belyaev@itep.ru
 *  @date 27 Apr 2002 
 */
// ============================================================================
DECLARE_COMPONENT( CaloSelectorOR )
// ============================================================================
/*  Standard constructor
 *  @see GaudiTool
 *  @see  AlgTool 
 *  @see IAlgTool 
 *  @param type   tool type (?)
 *  @param name   tool name 
 *  @param parent tool parent   
 */
// ============================================================================
CaloSelectorOR::CaloSelectorOR
( const std::string& type,
  const std::string& name,
  const IInterface* parent )
  : GaudiTool ( type, name , parent ) 
{
  declareInterface<ICaloClusterSelector> (this);
}
// ============================================================================
/*  standard initialization of the tool
 *  @see IAlgTool 
 *  @see AlgTool 
 *  @see GaudiTool 
 *  @return status code 
 */
// ============================================================================
StatusCode CaloSelectorOR::initialize () 
{
  // initialize the base class
  StatusCode sc = GaudiTool::initialize() ;
  if( sc.isFailure() ) 
    { return Error("Could not initialize the base class GaudiTool",sc);}
  // locate selectors 
  std::transform( m_selectorsTypeNames.begin(), m_selectorsTypeNames.end(),
                  std::back_inserter(m_selectors),
                  [&](const std::string& name)
                  { return  tool<ICaloClusterSelector>( name, this ); } );
  ///
  return StatusCode::SUCCESS ;
}
// ============================================================================
/*  standard finalization  of the tool
 *  @see IAlgTool 
 *  @see AlgTool 
 *  @see GaudiTool 
 *  @return status code 
 */
// ============================================================================
StatusCode CaloSelectorOR::finalize   () 
{
  // clear containers 
  m_selectors          .clear() ;
  m_selectorsTypeNames .clear() ;
  // finalize the base class 
  return GaudiTool::finalize () ;
}
// ============================================================================
/*  "select"/"preselect" method 
 *  @see ICaloClusterSelector
 *  @param  cluster pointer to calo cluster object to be selected 
 *  @return true if cluster is selected
 */
// ============================================================================
bool CaloSelectorOR::select     
( const LHCb::CaloCluster* cluster ) const { return (*this) ( cluster ) ; }
// ============================================================================
/** "select"/"preselect" method (functor interface)
 *  @see ICaloClusterSelector
 *  @param  cluster pointer to calo cluster object to be selected 
 *  @return true if cluster is selected
 */
// ============================================================================
bool CaloSelectorOR::operator() ( const LHCb::CaloCluster* cluster ) const
{
  return std::any_of( m_selectors.begin(), m_selectors.end(),
                      [&](typename Selectors::const_reference s)
                      { return (*s)(cluster); } );
}
// ============================================================================
// The END 
// ============================================================================
