#ifndef CALORECO_SUBCLUSTERSELECTOR3x3_H
#define CALORECO_SUBCLUSTERSELECTOR3x3_H 1
// Include files
#include "SubClusterSelectorBase.h"
#include "CaloUtils/CellMatrix3x3.h"

/** @class SubClusterSelector3x3 SubClusterSelector3x3.h
 *
 *  The simplest concrete "subcluster selector" -
 *  it just tag/select digits which form
 *  3x3 sub-matrix centered with the seed cells
 *
 *  @author Vanya Belyaev Ivan.Belyaev@itep.ru
 *  @date   07/11/2001
 */

class SubClusterSelector3x3  : public SubClusterSelectorBase{
public:

  StatusCode initialize() override;

  /** The main processing method
   *
   *  @see ICaloSubClusterTag
   *
   *  Error codes:
   *    - 225   cluster points to NULL
   *    - 226   empty cell/digit container for given cluster
   *    - 227   SeedCell is not found
   *    - 228   Seed points to NULL
   *
   *  @see ICaloClusterTool
   *  @see ICaloSubClusterTag
   *
   *  @param cluster pointer to CaloCluster object to be processed
   *  @return status code
   */

  StatusCode tag ( LHCb::CaloCluster* cluster) const override;

  /** The main method - "undo" of tagging from "tag" method
   *  @see ICaloSubClusterTag
   *  @param cluster pointer to ClaoCluster object to be untagged
   *  @return status code
   */
  StatusCode untag      ( LHCb::CaloCluster* cluster ) const override;

  /** Standard Tool Constructor
   *  @param type type of the tool (useless ? )
   *  @param name name of the tool
   *  @param parent the tool parent
   */
  SubClusterSelector3x3( const std::string& type   ,
                         const std::string& name   ,
                         const IInterface*  parent );

private:

  CellMatrix3x3        m_matrix;

};

#endif // SUBCLUSTERSELECTOR3x3_H
