#ifndef PRCOUNTER_H
#define PRCOUNTER_H 1

// Include files
// from Gaudi
#include "GaudiAlg/GaudiTool.h"

#include "Event/MCParticle.h"
#include "Event/Track.h"
#include "Linker/LinkerTool.h"
#include "PrKernel/IPrCounter.h"
//#include "MCInterfaces/IIdealStateCreator.h"
#include "TrackInterfaces/ITrackExtrapolator.h"
#include "GaudiKernel/ToolHandle.h"
#include "TrackInterfaces/ITrackSelector.h"


/** @class PrCounter2 PrCounter2.h
 *  This is a counter for track efficiency measurement.
 *
 *  @author Olivier Callot
 *  @date   2005-06-10
 *  Modified by Wenbin Qian for the VP Pat Efficiency
 *  @adpate to A-Team framework 2007-08-20 SHM
 */


class PrCounter2 : virtual public IPrCounter, public GaudiTool {
public:

  /// Standard constructor
  PrCounter2( const std::string& type,
              const std::string& name,
              const IInterface* parent);

  StatusCode initialize() override;
  StatusCode finalize() override;

  void initEvent(const IHistoTool* htool, const int nPV) override;

  int countAndPlot(const IHistoTool* htool,const LHCb::MCParticle* part, std::vector<bool> flags,
                       std::vector<LHCb::LHCbID>& ids, const int nPV) override;

  void addSelection ( std::string name, bool writeHisto ) override;

  void printStatistics() override;

protected:

private:
  typedef LinkerTool<LHCb::Track, LHCb::MCParticle> MyAsct;
  typedef MyAsct::DirectType            Table;
  typedef MyAsct::DirectType::Range     Range;
  typedef Table::iterator               iterator;

  typedef MyAsct::InverseType           InvTable;
  typedef InvTable::Range               InvRange;
  typedef InvTable::iterator            InvIterator;

  MyAsct*         m_link = nullptr;
  const InvTable* m_invTable;

  ITrackExtrapolator* m_extrapolator = nullptr;
  //IIdealStateCreator* m_idealStateCreator;

  std::string  m_title;
  unsigned int m_titleSize;

  bool         m_validData;

  // total variables
  int          m_totTrack;                ///< Total number of tracks processed
  int          m_totGhost;                ///< Total number of ghosts
  int          m_totTrackTrigger;                ///< Total number of tracks processed
  int          m_totGhostTrigger;                ///< Total number of ghosts
  double       m_fracGhost;
  double       m_nEvent;

  std::vector<std::string> m_name;    ///< Name of the sub-counters
  std::vector<bool>        m_writeHisto;    ///< Make histograms for this container
  std::vector<int>         m_wanted;  ///< Nb MC tracks measurable.
  std::vector<int>         m_counted; ///< counters for statistics
  std::vector<int>         m_clone;   ///< counters for clones
  std::vector<double>      m_purity;  ///< Sum of purity (linker weight)
  std::vector<double>      m_hitEff;  ///< Sum of hitEfficiency

  ToolHandle<ITrackSelector> m_selector;
};

#endif // PRCOUNTER_H
