// Include files

// local
#include "Event/ProcStatus.h"
#include "AddToProcStatus.h"

//-----------------------------------------------------------------------------
// Implementation file for class : AddToProcStatus
//
// 2011-06-15 : Patrick Koppenburg
//-----------------------------------------------------------------------------

// Declaration of the Algorithm Factory
DECLARE_COMPONENT( AddToProcStatus )

//=============================================================================
// Initialization
//=============================================================================
StatusCode AddToProcStatus::initialize()
{
  StatusCode sc = GaudiAlgorithm::initialize(); // must be executed first
  if ( sc.isFailure() ) return sc;  // error printed already by GaudiAlgorithm

  if ( m_reason.empty()    ) { return Error("Please provide a Reason"); }
  if ( m_subsystem.empty() ) { return Error("Please provide a Subsystem"); }

  info() << "Whenever called will add the following to ProcStatus:" << endmsg;
  info() << "  Algorithm: " << name() << ", Subsystem: " << m_subsystem.value()
         << ", Status: " << m_status.value() << ", Abort: " << m_abort.value() << endmsg ;


  if ( msgLevel(MSG::DEBUG) ) debug() << "==> Initialize" << endmsg;

  return sc;
}

//=============================================================================
// Main execution
//=============================================================================
StatusCode AddToProcStatus::execute()
{
  if ( msgLevel(MSG::DEBUG) ) debug() << "==> Execute" << endmsg;

  LHCb::ProcStatus* procStat =
    getOrCreate<LHCb::ProcStatus,LHCb::ProcStatus>(LHCb::ProcStatusLocation::Default);

  // give some indication that we had to skip this event
  // (ProcStatus returns zero status for us in cases where we don't
  // explicitly add a status code)
  procStat->addAlgorithmStatus( name(), m_subsystem, m_reason, m_status, m_abort );
  return Warning("Processing aborted", StatusCode::SUCCESS, 0);
}

//=============================================================================
