#ifndef TIMINGTUPLE_H
#define TIMINGTUPLE_H 1

#include <algorithm>
#include <atomic>

// from Gaudi
#include "GaudiAlg/GaudiTupleAlg.h"
#include "GaudiAlg/Consumer.h"
#include "GaudiKernel/Memory.h"
#include "GaudiAlg/ISequencerTimerTool.h"

#include "Event/ODIN.h"
#include "Event/RecSummary.h"

class ISequencerTimerTool ;

/** @class TimingTuple TimingTuple.h
 *
 *  Fill a Tuple with timing, memory and some event variables
 *
 *  @author Patrick Koppenburg
 *  @date   2010-08-18
 */
class TimingTuple final
: public Gaudi::Functional::Consumer<void(const LHCb::ODIN&, const LHCb::RecSummary&),
                                     Gaudi::Functional::Traits::BaseClass_t<GaudiTupleAlg> >
{

public:

  /// Standard constructor
  TimingTuple( const std::string& name, ISvcLocator* pSvcLocator );

  StatusCode initialize() override;    ///< Algorithm initialization
  void operator()(const LHCb::ODIN&, const LHCb::RecSummary&) const override ; ///< Algorithm execution

private:

  ISequencerTimerTool* m_timerTool = nullptr; ///< timer tool
  int m_timer = 0; ///< timer index
  mutable std::atomic<unsigned long long> m_evtCounter = { 0 };

  template <class TYPE>
  void fillTuple(Tuple& tuple, const std::string& var, const TYPE number) const
  {
    if (msgLevel(MSG::VERBOSE))
      verbose() << "Filling " << var << " with " << number << endmsg ;
    tuple->column( var , number );
  }

};
#endif // TIMINGTUPLE_H
