
//-----------------------------------------------------------------------------
/** @file RichRecHistoAlgBase.cpp
 *
 *  Implementation file for RICH reconstruction monitor
 *  algorithm base class : Rich::Rec::HistoAlgBase
 *
 *  @author Chris Jones    Christopher.Rob.Jones@cern.ch
 *  @date   2005/01/13
 */
//-----------------------------------------------------------------------------

// local
#include "RichFutureRecBase/RichRecHistoAlgBase.h"

// ============================================================================
// Force creation of templated class
#include "RichRecBase.icpp"
template class Rich::Future::Rec::CommonBase<Rich::Future::HistoAlgBase> ;
// ============================================================================

// ============================================================================
// Standard constructor
// ============================================================================
Rich::Future::Rec::HistoAlgBase::HistoAlgBase( const std::string& name,
                                       ISvcLocator* pSvcLocator )
  : Rich::Future::HistoAlgBase   ( name, pSvcLocator ),
    Rich::Future::Rec::CommonBase<Rich::Future::HistoAlgBase> ( this )
{
}
// ============================================================================

// ============================================================================
// Initialise
// ============================================================================
StatusCode Rich::Future::Rec::HistoAlgBase::initialize()
{
  // Initialise base class
  StatusCode sc = Rich::Future::HistoAlgBase::initialize();
  if ( sc.isFailure() )
    return Error( "Failed to initialise Rich::HistoAlgBase", sc );

  // Common initialisation
  sc = initialiseRichReco();
  if ( sc.isFailure() )
    return Error( "Failed to initialise Rich::RecBase", sc );

  return sc;
}
// ============================================================================

// ============================================================================
// Main execute method
// ============================================================================
StatusCode Rich::Future::Rec::HistoAlgBase::execute()
{
  // All algorithms should re-implement this method
  return Error ( "Default Rich::RecHistoAlgBase::execute() called !!" );
}
// ============================================================================

// ============================================================================
// Finalize
// ============================================================================
StatusCode Rich::Future::Rec::HistoAlgBase::finalize()
{
  // Common finalisation
  const StatusCode sc = finaliseRichReco();
  if ( sc.isFailure() ) return Error( "Failed to finalise Rich::RecBase", sc );

  // Finalize base class
  return Rich::Future::HistoAlgBase::finalize();
}
// ============================================================================
