
//-----------------------------------------------------------------------------
/** @file RichRecHistoToolBase.cpp
 *
 *  Implementation file for RICH reconstruction tool base class : RichRecHistoToolBase
 *
 *  @author Chris Jones    Christopher.Rob.Jones@cern.ch
 *  @date   2005/01/13
 */
//-----------------------------------------------------------------------------

// local
#include "RichFutureRecBase/RichRecHistoToolBase.h"

// ============================================================================
// Force creation of templated class
#include "RichRecBase.icpp"
template class Rich::Future::Rec::CommonBase<Rich::Future::HistoToolBase> ;
// ============================================================================

// ============================================================================
// Standard constructor
// ============================================================================
Rich::Future::Rec::HistoToolBase::HistoToolBase( const std::string& type,
                                                 const std::string& name,
                                                 const IInterface* parent )
  : Rich::Future::HistoToolBase ( type, name, parent ),
    Rich::Future::Rec::CommonBase<Rich::Future::HistoToolBase> ( this )
{
}
// ============================================================================

// ============================================================================
// Initialise
// ============================================================================
StatusCode Rich::Future::Rec::HistoToolBase::initialize()
{
  // Initialise base class
  StatusCode sc = Rich::Future::HistoToolBase::initialize();
  if ( sc.isFailure() )
    return Error( "Failed to initialise RichHistoToolBase", sc );

  // Common initialisation
  sc = initialiseRichReco();
  if ( sc.isFailure() )
    return Error( "Failed to initialise Rich::RecBase", sc );

  return sc;
}
// ============================================================================

// ============================================================================
// Finalise
// ============================================================================
StatusCode Rich::Future::Rec::HistoToolBase::finalize()
{
  // Common finalisation
  const StatusCode sc = finaliseRichReco();
  if ( sc.isFailure() ) return Error( "Failed to finalise Rich::RecBase", sc );

  // base class finalize
  return Rich::Future::HistoToolBase::finalize();
}
// ============================================================================
