
//-----------------------------------------------------------------------------
/** @file RichRecTupleToolBase.cpp
 *
 *  Implementation file for RICH reconstruction tool base class : RichRecTupleToolBase
 *
 *  @author Chris Jones    Christopher.Rob.Jones@cern.ch
 *  @date   2005/01/13
 */
//-----------------------------------------------------------------------------

// local
#include "RichFutureRecBase/RichRecTupleToolBase.h"

// ============================================================================
// Force creation of templated class
#include "RichRecBase.icpp"
template class Rich::Future::Rec::CommonBase<Rich::Future::TupleToolBase> ;
// ============================================================================

// ============================================================================
// Standard constructor
// ============================================================================
Rich::Future::Rec::TupleToolBase::TupleToolBase( const std::string& type,
                                                 const std::string& name,
                                                 const IInterface* parent )
  : Rich::Future::TupleToolBase ( type, name, parent ),
    Rich::Future::Rec::CommonBase<Rich::Future::TupleToolBase> ( this )
{
}
// ============================================================================

// ============================================================================
// Initialise
// ============================================================================
StatusCode Rich::Future::Rec::TupleToolBase::initialize()
{
  // Initialise base class
  const StatusCode sc = Rich::Future::TupleToolBase::initialize();
  if ( sc.isFailure() )
    return Error( "Failed to initialise Rich::TupleToolBase", sc );

  // Common initialisation
  return initialiseRichReco();
}
// ============================================================================

// ============================================================================
// Finalise
// ============================================================================
StatusCode Rich::Future::Rec::TupleToolBase::finalize()
{
  // Common finalisation
  const StatusCode sc = finaliseRichReco();
  if ( sc.isFailure() ) return Error( "Failed to finalise Rich::RecBase", sc );

  // base class finalize
  return Rich::Future::TupleToolBase::finalize();
}
// ============================================================================
