#ifndef LSADAPTPVFITTER_H
#define LSADAPTPVFITTER_H
// from Gaudi
#include "GaudiAlg/GaudiTool.h"
// Interfaces
#include "IPVFitter.h"
#include "PVUtils.h"
// Track info
#include "Event/Track.h"
#include "Event/State.h"
#include "Event/RecVertex.h"
#include "TrackInterfaces/ITrackExtrapolator.h"

class LSAdaptPVFitter : public extends<GaudiTool, IPVFitter> {

public:
  // Standard constructor
  LSAdaptPVFitter(const std::string& type,
                  const std::string& name,
                  const IInterface* parent);
  // Fitting
  StatusCode fitVertex(const Gaudi::XYZPoint& seedPoint,
                       const std::vector<const LHCb::Track*>& tracks,
                       LHCb::RecVertex& vtx,
                       std::vector<const LHCb::Track*>& tracks2remove) const override;
private:
  Gaudi::Property<int>    m_minTr { this, "MinTracks", 5, "Minimum number of tracks in vertex"};
  Gaudi::Property<int>    m_maxIterations { this, "Iterations", 50, "Maximum number of iterations"};
  Gaudi::Property<int>    m_minIter { this, "minIter", 5, "Minimum number of iterations"};
  Gaudi::Property<double> m_maxChi2 { this, "maxChi2", 400.0, "Chi2 of completely wrong tracks" };
  Gaudi::Property<double> m_maxDeltaZ { this, "maxDeltaZ", 0.0005 * Gaudi::Units::mm, "Fit convergence condition delta z"};
  Gaudi::Property<double> m_maxDeltaChi2NDoF { this, "maxDeltaChi2NDoF", 0.002, "Fit convergence condition delta chi2/ndof if not converged by dz" };
  Gaudi::Property<double> m_minTrackWeight { this,  "acceptTrack", 0.00000001, "Value of the Tukey's weight to accept a track" };
  Gaudi::Property<double> m_trackMaxChi2 {this, "trackMaxChi2", 9., "maximum chi2 track to accept track in PV fit" };
  Gaudi::Property<double> m_trackMaxChi2Remove { this, "trackMaxChi2Remove", 25., "Max chi2 tracks to be removed from next PV search"};
  double m_trackChi = 3.;      // sqrt of m_trackMaxChi2

  // Extrapolators
  ToolHandle<ITrackExtrapolator> m_linExtrapolator { this, "LinearExtrapolator", "TrackLinearExtrapolator"};
  ToolHandle<ITrackExtrapolator> m_fullExtrapolator{ this, "FullExtrapolator","TrackMasterExtrapolator" };

  // Least square iterative PV fit
  StatusCode fit(LHCb::RecVertex& vtx, std ::vector<PVTrack*>& pvTracks,
                 std::vector<const LHCb::Track*>& tracks2remove) const;
  // Add track for PV
  StatusCode addTrackForPV(const LHCb::Track* str,std::vector<PVTrack>& pvTracks,
                           double zseed) const;

  void initVertex(PVTracks& pvTracks,
                  PVVertex& pvVertex,
                  const Gaudi::XYZPoint seedPoint) const;

  // Prepare hessian matrix and vectorD0
  void prepareVertex(LHCb::RecVertex& vtx,
                    PVTrackPtrs& pvTracks,
                    Gaudi::SymMatrix3x3& hess,
                    ROOT::Math::SVector<double,3>& d0vec,
                    int iter) const;
  // Extrapolation
  StatusCode trackExtrapolate(PVTrack* pvTrack,
                              const LHCb::RecVertex& vtx) const;
  // Add track to fit
  void addTrack(PVTrack* pTrack,
                Gaudi::SymMatrix3x3& hessian,
                ROOT::Math::SVector<double,3>& vectorD0) const;
  // Remove track from fit
  void removeTrack(PVTrack* pTrack,
                Gaudi::SymMatrix3x3& hessian,
                ROOT::Math::SVector<double,3>& vectorD0) const;
  // Add subtrack
  void addsubTrack(PVTrack* pTrack,
                   Gaudi::SymMatrix3x3& hessian,
                   ROOT::Math::SVector<double,3>& vectorD0,
                   double invs) const;
  // Update matrices and vectors
  StatusCode outVertex(LHCb::RecVertex& vtx,
                       PVTrackPtrs& pvTracks,
                       Gaudi::SymMatrix3x3& hess,
                       ROOT::Math::SVector<double,3>& d0vec) const;
  // Set current chi2
  void setChi2(LHCb::RecVertex& vtx,
               PVTrackPtrs& pvTracks) const;

  // Get Tukey's weight
  double getTukeyWeight(double trchi2, int iter) const;

  void printTracks(PVTrackPtrs& pvTracks) const;

};
#endif // LSADAPTPVFITTER_H
