#ifndef PATPV_PATPV3D_H
#define PATPV_PATPV3D_H 1

// Include files
// -------------
// From Gaudi
#include "GaudiAlg/Transformer.h"
// Interfaces
#include "TrackInterfaces/IPVOfflineTool.h"
// Local
#include "Event/PrimaryVertex.h"
#include "Event/RecVertex.h"

/** @class PatPV3D PatPV3D.h
 *  Algorithm to find the primary vertices at the HLT.
 *
 *  @author Eduardo Rodrigues
 *  @author Sebastien Ponce
 */

//-----------------------------------------------------------------------------

typedef Gaudi::Functional::MultiTransformerFilter<
    std::tuple<LHCb::RecVertices,LHCb::PrimaryVertices>(const LHCb::Tracks&)
  > PatPV3DBaseClass;

class PatPV3D : public PatPV3DBaseClass {
public:
  /// Standard constructor
  PatPV3D(const std::string& name, ISvcLocator* pSvcLocator);

  /// Algorithm execution
  std::tuple<bool, LHCb::RecVertices,LHCb::PrimaryVertices> operator()(const LHCb::Tracks&) const override;

private:
  Gaudi::Property<bool> m_refitpv { this,  "RefitPV", false, "Flag to refit PVs when converting to type PrimaryVertex" };
  ToolHandle<IPVOfflineTool> m_pvsfit {"PVOfflineTool",this};   // PV fitting tool

};
#endif // PATPV_PATPV3D_H
