#ifndef _ITHitExpectation_H
#define _ITHitExpectation_H

/** @class ITHitExpectation ITHitExpectation.h
 *
 * Implementation of ITHitExpectation tool
 * see interface header for description
 *
 *  @author M.Needham
 *  @date   22/5/2007
 */


#include "THitExpectation.h"
#include "Kernel/ISTChannelIDSelector.h"
#include <string>



namespace LHCb{
  class Track;
  class STChannelID;
  class LHCbID;
}

class DeSTDetector;


namespace Tf {
  namespace Tsa {
    class IITExpectedHits;
  }
}

class ITHitExpectation final : public THitExpectation {

public:

  /** constructor */
  using THitExpectation::THitExpectation;

  /** intialize */
  StatusCode initialize() override;

  /** Returns number of hits expected, from zFirst to inf
  *
  *  @param aTrack Reference to the Track to test
  *
  *  @return number of hits expected
  */
  unsigned int nExpected ( const LHCb::Track& aTrack ) const override;


  /** Returns number of hits expected
  *
  *  @param aTrack Reference to the Track to test
  *
  *  @return Info info including likelihood
  */
  IHitExpectation::Info expectation( const LHCb::Track& aTrack ) const override;

  /** Collect all the expected hits
   *
   * @param aTrack Reference to the Track to test
   * @param hits collected lhcbIDs
   *
   **/
  void collect(const LHCb::Track& aTrack,
               std::vector<LHCb::LHCbID>& ids ) const override;

private:

  typedef std::vector<Tf::Tsa::IITExpectedHits::ITPair> ITPairs;

  bool select(const LHCb::STChannelID& chan) const;

  int boxToSector(const LHCb::STChannelID& chan) const;

  DeSTDetector* m_itDet = nullptr;
  Tf::Tsa::IITExpectedHits* m_expectedITHits = nullptr;
  ISTChannelIDSelector* m_selector = nullptr;

  Gaudi::Property<std::string> m_selectorType { this,  "SelectorType", "STSelectChannelIDByElement" };
  Gaudi::Property<std::string> m_selectorName { this,  "SelectorName", "ALL" };
  Gaudi::Property<bool> m_allStrips { this,  "allStrips", false };

};


inline bool ITHitExpectation::select(const LHCb::STChannelID& chan) const{
  return m_selector == 0 ? true : m_selector->select(chan);
}


#endif
